# file_mate/tests/test_utils.py

import pytest
from file_mate import utils
import os

# Create a dummy test file
TEST_FILE = "test_file.txt"
TEST_DIR = "test_dir"


@pytest.fixture(scope="module", autouse=True)
def setup_teardown():
    """Setup and teardown method."""
    with open(TEST_FILE, 'w') as f:
        f.write("Test content")
    os.makedirs(TEST_DIR, exist_ok=True)
    yield
    os.remove(TEST_FILE)

    #Remove directory with content
    def remove_directory(path):
         for item in os.listdir(path):
             item_path = os.path.join(path,item)
             if os.path.isfile(item_path):
                 os.remove(item_path)
             elif os.path.isdir(item_path):
                   remove_directory(item_path)
         os.rmdir(path)

    remove_directory(TEST_DIR) if os.path.exists(TEST_DIR) else None


def test_validate_file_exists_success():
    utils.validate_file_exists(TEST_FILE)


def test_validate_file_exists_fail():
    with pytest.raises(FileNotFoundError):
        utils.validate_file_exists("non_existent_file.txt")


def test_get_file_type():
    file_type = utils.get_file_type(TEST_FILE)
    assert "text" in file_type.lower()


def test_validate_output_dir_create():
    output_file = os.path.join(TEST_DIR, "new_test", "test.txt")
    utils.validate_output_dir(output_file)
    assert os.path.exists(os.path.dirname(output_file))
    os.rmdir(os.path.dirname(output_file))


def test_validate_output_dir_exists():
    output_file = os.path.join(TEST_DIR, "test.txt")
    utils.validate_output_dir(output_file)
    assert os.path.exists(TEST_DIR)



# file_mate/tests/test_manipulators.py
import pytest
import os
from file_mate import manipulators
from PIL import Image
from reportlab.pdfgen import canvas  # Add this import
from reportlab.lib.pagesizes import letter

# Create dummy image file
TEST_IMAGE = "test_image.png"
TEST_PDF = "test.pdf"  # New test pdf
TEST_ROTATED_IMAGE = "rotated_image.png"
TEST_RESIZED_IMAGE = "resized_image.png"
TEST_GRAY_IMAGE = "gray_image.png"


@pytest.fixture(scope="module", autouse=True)
def setup_teardown():
    """Set up the tests by creating dummy files"""
    with Image.new("RGB", (60, 30), color="red") as img:
        img.save(TEST_IMAGE)
    # No need for img.close() as the context manager handles it

    c = canvas.Canvas(TEST_PDF, pagesize=letter)  # New pdf object
    c.drawString(100, 100, "Test")  # Dummy text
    c.save()

    yield

    # Ensure files are removed
    for file in [TEST_IMAGE, TEST_PDF, TEST_ROTATED_IMAGE, TEST_RESIZED_IMAGE, TEST_GRAY_IMAGE]:
        if os.path.exists(file):
            os.remove(file)


def test_resize_image_success_with_scale():
    output_file = TEST_RESIZED_IMAGE
    manipulators.resize_image(TEST_IMAGE, output_file, scale=0.5)
    assert os.path.exists(output_file)
    with Image.open(output_file) as img:
       assert img.size == (30, 15)


def test_resize_image_success_with_width_and_height():
    output_file = TEST_RESIZED_IMAGE
    manipulators.resize_image(TEST_IMAGE, output_file, width=100, height=200)
    assert os.path.exists(output_file)
    with Image.open(output_file) as img:
        assert img.size == (100, 200)


def test_resize_image_fail_file_not_found():
    with pytest.raises(FileNotFoundError):
        manipulators.resize_image("not_exists.png", "out.png", width=200, height=100)


def test_resize_image_fail_invalid_image():
    with pytest.raises(ValueError):
        manipulators.resize_image(TEST_PDF, "out.png", width=100, height=200)


def test_resize_image_fail_no_resize_options():
    with pytest.raises(ValueError):  # Changed
        manipulators.resize_image(TEST_IMAGE, "out.png")


def test_rotate_image_success():
    output_file = TEST_ROTATED_IMAGE
    manipulators.rotate_image(TEST_IMAGE, output_file, degrees=90)
    assert os.path.exists(output_file)
    with Image.open(output_file) as img:
        assert img.size == (30, 60)


def test_rotate_image_fail_file_not_found():
    with pytest.raises(FileNotFoundError):
        manipulators.rotate_image("not_exists.png", "out.png", degrees=90)


def test_rotate_image_fail_invalid_image():
    with pytest.raises(ValueError):
        manipulators.rotate_image(TEST_PDF, "out.png", degrees=90)


def test_rotate_image_fail_invalid_degree():
    with pytest.raises(ValueError, match="Error: Rotation must be one of 90, 180 or 270. Got 45"):
        manipulators.rotate_image(TEST_IMAGE, "out.png", degrees=45)


def test_grayscale_image_success():
    output_file = TEST_GRAY_IMAGE
    manipulators.grayscale_image(TEST_IMAGE, output_file)
    assert os.path.exists(output_file)


def test_grayscale_image_fail_file_not_found():
    with pytest.raises(FileNotFoundError):
        manipulators.grayscale_image("not_exists.png", "out.png")


def test_grayscale_image_fail_invalid_image():
    with pytest.raises(ValueError):
        manipulators.grayscale_image(TEST_PDF, "out.png")



# file_mate/tests/test_converters.py
import pytest
import os
from file_mate import converters
from PIL import Image
from reportlab.pdfgen import canvas  # Add this line
from reportlab.lib.pagesizes import letter
from pathlib import Path

# Create dummy image files
TEST_IMAGE = "test_image.png"
TEST_PDF = "test.pdf"
TEST_IMG_PDF = "test_img_pdf.pdf"
TEST_MERGED_PDF = "merged.pdf"
TEST_DIR = Path("test_dir")


@pytest.fixture(scope="module", autouse=True)
def setup_teardown():
    """Set up the tests by creating dummy files"""
    img = Image.new('RGB', (60, 30), color='red')
    img.save(TEST_IMAGE)

    # Create PDF with an embedded image
    c = canvas.Canvas(TEST_PDF, pagesize=letter)
    width, height = letter
    c.drawImage(TEST_IMAGE, 0, 0, width=width / 2, height=height / 2)  # Embed the image
    c.save()

    # Create another PDF for merging
    c = canvas.Canvas(TEST_IMG_PDF, pagesize=letter)
    c.drawImage(TEST_IMAGE, 0, 0, width=width, height=height)
    c.save()

    os.makedirs(TEST_DIR, exist_ok=True)

    yield

    os.remove(TEST_IMAGE)
    os.remove(TEST_PDF)
    os.remove(TEST_IMG_PDF)
    os.remove(TEST_MERGED_PDF) if os.path.exists(TEST_MERGED_PDF) else None
    os.rmdir(TEST_DIR)

    #Remove directory with content
    def remove_directory(path):
         for item in os.listdir(path):
             item_path = os.path.join(path,item)
             if os.path.isfile(item_path):
                 os.remove(item_path)
             elif os.path.isdir(item_path):
                   remove_directory(item_path)
         os.rmdir(path)

    remove_directory(TEST_DIR) if os.path.exists(TEST_DIR) else None


def test_convert_image_success():
    output_file = "converted_image.jpg"
    converters.convert_image(
        TEST_IMAGE, output_file, "jpeg"
    )  # changed from jpg to jpeg
    assert os.path.exists(output_file)
    os.remove(output_file)

    target_format = "jpeg"
    if target_format:
        target_format = target_format.lower()  # Add this line


def test_convert_image_fail_file_not_found():
    with pytest.raises(FileNotFoundError):
        converters.convert_image("not_exists.png", "out.png")


def test_convert_image_fail_invalid_image():
    with pytest.raises(ValueError):
        converters.convert_image(TEST_PDF, "out.png")


def test_convert_image_fail_invalid_format():
    with pytest.raises(RuntimeError):
        output_file = "converted_image.invalid"
        converters.convert_image(TEST_IMAGE, output_file)


def test_convert_image_to_pdf_success():
    output_file = "converted_image.pdf"
    converters.convert_image_to_pdf(TEST_IMAGE, output_file)
    assert os.path.exists(output_file)
    os.remove(output_file)


def test_convert_image_to_pdf_fail_file_not_found():
    with pytest.raises(FileNotFoundError):
        converters.convert_image_to_pdf("not_exists.png", "out.pdf")


def test_convert_image_to_pdf_fail_invalid_image():
    with pytest.raises(ValueError):
        converters.convert_image_to_pdf(TEST_PDF, "out.pdf")


def test_convert_pdf_to_images_success():
    output_dir = TEST_DIR / "pdf_to_images"
    converters.convert_pdf_to_images(TEST_PDF, output_dir)
    assert len(os.listdir(output_dir)) > 0  # Ensure at least one image is created
    for file in os.listdir(output_dir):
        os.remove(output_dir / file)
    os.rmdir(output_dir)


def test_convert_pdf_to_images_fail_file_not_found():
    with pytest.raises(FileNotFoundError):
        converters.convert_pdf_to_images("not_exists.pdf", "test_dir")


def test_convert_pdf_to_images_fail_invalid_pdf():
    with pytest.raises(ValueError):
        converters.convert_pdf_to_images(TEST_IMAGE, "test_dir")


def test_merge_pdf_success():
    output_file = TEST_MERGED_PDF
    converters.merge_pdf([TEST_PDF, TEST_IMG_PDF], output_file)
    assert os.path.exists(output_file)


def test_merge_pdf_fail_file_not_found():
    with pytest.raises(FileNotFoundError):
        converters.merge_pdf(["test1.pdf", "test2.pdf"], "merged_out.pdf")


def test_merge_pdf_fail_invalid_pdf():
    with pytest.raises(ValueError):
        converters.merge_pdf([TEST_IMAGE, TEST_PDF], "merged_out.pdf")



# file_mate/utils.py

import os
import magic

def validate_file_exists(file_path):
    """Checks if a file exists at the specified path."""
    if not os.path.exists(file_path):
        raise FileNotFoundError(f"Error: File not found: {file_path}")

def get_file_type(file_path):
    """Uses libmagic to get a descriptive file type."""
    try:
        m = magic.Magic()
        return m.from_file(file_path)
    except Exception as e:
        raise RuntimeError(f"Error determining file type: {e}")

def validate_image_format(file_path):
    """Validates if the file is an image."""
    file_type = get_file_type(file_path)
    if "image" not in file_type.lower():
      raise ValueError(f"Error: File {file_path} is not an image")

def validate_pdf_format(file_path):
    """Validates if the file is a PDF"""
    file_type = get_file_type(file_path)
    if "pdf" not in file_type.lower():
      raise ValueError(f"Error: File {file_path} is not a pdf")


def validate_output_dir(output_path):
    """Checks if the output directory exists or create it"""
    directory = os.path.dirname(output_path)
    if directory and not os.path.exists(directory):
        os.makedirs(directory, exist_ok=True)


# file_mate/tests/test_info.py

import pytest
import os
from file_mate import info
from PIL import Image
from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import letter

# Create dummy image file
TEST_IMAGE = "test_image.png"
TEST_PDF = "test.pdf"


@pytest.fixture(scope="module", autouse=True)
def setup_teardown():
    """Set up the tests by creating dummy files"""
    img = Image.new("RGB", (60, 30), color="red")
    img.save(TEST_IMAGE)

    c = canvas.Canvas(TEST_PDF, pagesize=letter)
    width, height = letter
    c.drawString(100, height - 100, "test")
    c.save()

    yield
    os.remove(TEST_IMAGE)
    os.remove(TEST_PDF)


def test_get_file_info_image_success():
    file_info = info.get_file_info(TEST_IMAGE)
    assert "type" in file_info
    assert "size" in file_info
    assert "size_bytes" in file_info
    assert "dimensions" in file_info
    assert file_info["dimensions"] == "60x30"


def test_get_file_info_pdf_success():
    file_info = info.get_file_info(TEST_PDF)
    assert "type" in file_info
    assert "size" in file_info
    assert "size_bytes" in file_info
    assert "page_count" in file_info
    assert file_info["page_count"] == 1


def test_get_file_info_fail_file_not_found():
    with pytest.raises(FileNotFoundError):
        info.get_file_info("non_existent.txt")



# file_mate/converters.py

from PIL import Image
from file_mate.utils import (
    validate_file_exists,
    validate_image_format,
    validate_output_dir,
)
import os
from pypdf import PdfReader, PdfWriter
from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import letter
from reportlab.lib.units import inch


def convert_image(input_file, output_file, target_format=None, quality=100):
    """Converts an image to a specified format.

    Args:
       input_file (str): The path of the input file.
       output_file (str): The path of the output file.
       target_format (str, optional): The target image format. Defaults to None.
       quality (int, optional): The quality of output image (0-100). Defaults to 100.
    """
    validate_file_exists(input_file)
    validate_image_format(input_file)
    validate_output_dir(output_file)

    if not target_format:
        target_format = os.path.splitext(output_file)[1][1:].lower()

    try:
        img = Image.open(input_file)
        img.save(output_file, format=target_format, quality=quality)

    except Exception as e:
        raise RuntimeError(f"Error converting image: {e}")


def convert_image_to_pdf(input_file, output_file):
    """Converts an image to a pdf

    Args:
        input_file (str): The path of the input file.
        output_file (str): The path of the output file.
    """
    validate_file_exists(input_file)
    validate_image_format(input_file)
    validate_output_dir(output_file)

    try:
        img = Image.open(input_file)
        c = canvas.Canvas(output_file, pagesize=letter)
        width, height = letter
        img_width, img_height = img.size

        # Calculate the position and size of the image to fit within the page.
        if (img_width > width or img_height > height):
            # Scale the image to fit within page limits while maintaining aspect ratio
            img_aspect_ratio = img_width / img_height
            page_aspect_ratio = width / height

            if img_aspect_ratio > page_aspect_ratio:
                new_width = width - (2 * 0.5 * inch)  # Subtract 0.5 inch padding from each side
                new_height = new_width / img_aspect_ratio
            else:
                new_height = height - (2 * 0.5 * inch)
                new_width = new_height * img_aspect_ratio

            x = (width - new_width) / 2
            y = (height - new_height) / 2
            c.drawImage(input_file, x, y, width=new_width, height=new_height)
        else:
            x = (width - img_width) / 2
            y = (height - img_height) / 2
            c.drawImage(input_file, x, y, width=img_width, height=img_height)

        c.save()
    except Exception as e:
        raise RuntimeError(f"Error converting image to PDF: {e}")


def convert_pdf_to_images(input_pdf, output_dir, target_format="png"):
    """Converts a PDF to series of images

    Args:
        input_pdf (str): The path of the input PDF file.
        output_dir (str): The output directory path to save the images
        target_format (str, optional): The target image format for the output images. Default is png.
    """
    validate_file_exists(input_pdf)
    from file_mate.utils import validate_pdf_format

    validate_pdf_format(input_pdf)

    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    try:
        images = []
        pdf = PdfReader(input_pdf)
        num_pages = len(pdf.pages)

        for i in range(num_pages):
            page = pdf.pages[i]
            if not page.images:
                continue  # Skip pages without images
            try:
                image = page.images[0]
                img_file = os.path.join(output_dir, f"page_{i+1}.{target_format}")
                with open(img_file, "wb") as f:
                    f.write(image.data)
                images.append(img_file)
            except IndexError:
                continue  # Skip pages without images
        if not images:
            raise RuntimeError("No images found in the PDF.")
    except Exception as e:
        raise RuntimeError(f"Error converting pdf to images: {e}")


def merge_pdf(input_pdfs, output_pdf):
    """Merges the input PDFs to an output PDF

    Args:
        input_pdfs (list): List of paths for the input PDF
        output_pdf (str): Output file path
    """

    for input_pdf in input_pdfs:
        validate_file_exists(input_pdf)
        from file_mate.utils import validate_pdf_format

        validate_pdf_format(input_pdf)

    validate_output_dir(output_pdf)
    merger = PdfWriter()

    try:
        for pdf_file in input_pdfs:
            merger.append(pdf_file)
        merger.write(output_pdf)
        merger.close()
    except Exception as e:
        raise RuntimeError(f"Error merging pdfs: {e}")



# file_mate/cli.py


import click
from file_mate.converters import (
    convert_image,
    convert_image_to_pdf,
    convert_pdf_to_images,
    merge_pdf,
)
from file_mate.info import get_file_info
from file_mate.manipulators import resize_image, rotate_image, grayscale_image


@click.group()
def cli():
    """Filemate is a command-line tool for image, pdf, and other file manipulations."""
    pass


@cli.group()
def convert():
    """Convert files between formats"""
    pass


@convert.command("image")
@click.argument("input_file")
@click.argument("output_file")
@click.option("--format", help="Target image format.")
@click.option("--quality", type=int, default=95, help="Image quality (0-100).")
def convert_image_cli(input_file, output_file, format, quality):
    """Converts an image to another format."""
    try:
        convert_image(input_file, output_file, format, quality)
        click.echo(f"Converted {input_file} to {output_file}")
    except Exception as e:
        click.echo(f"Error: {e}")


@convert.command("image-to-pdf")
@click.argument("input_file")
@click.argument("output_file")
def convert_image_to_pdf_cli(input_file, output_file):
    """Converts an image to a PDF."""
    try:
        convert_image_to_pdf(input_file, output_file)
        click.echo(f"Converted {input_file} to {output_file}")
    except Exception as e:
        click.echo(f"Error: {e}")


@convert.command("pdf-to-images")
@click.argument("input_pdf")
@click.argument("output_dir")
@click.option("--format", default="png", help="Target image format.")
def convert_pdf_to_images_cli(input_pdf, output_dir, format):
    """Converts a PDF to a series of images"""
    try:
        convert_pdf_to_images(input_pdf, output_dir, format)
        click.echo(f"Converted {input_pdf} to images in {output_dir}")
    except Exception as e:
        click.echo(f"Error: {e}")


@convert.command("merge-pdf")
@click.argument("input_pdfs", nargs=-1, type=click.Path(exists=True))
@click.argument("output_pdf")
def merge_pdf_cli(input_pdfs, output_pdf):
    """Merges the given input pdfs into an output pdf"""
    try:
        merge_pdf(input_pdfs, output_pdf)
        click.echo(f"Merged {input_pdfs} into {output_pdf}")
    except Exception as e:
        click.echo(f"Error: {e}")


@cli.group()
def manip():
    """Manipulate files"""
    pass


@manip.command("resize")
@click.argument("input_file")
@click.argument("output_file")
@click.option("--width", type=int, help="Target image width in pixels.")
@click.option("--height", type=int, help="Target image height in pixels.")
@click.option("--scale", type=float, help="Scale the image by this factor (0-1).")
def resize_image_cli(input_file, output_file, width, height, scale):
    """Resize image with scale or height and width"""
    try:
        resize_image(input_file, output_file, width, height, scale)
        click.echo(f"Resized {input_file} to {output_file}")
    except Exception as e:
        click.echo(f"Error: {e}")


@manip.command("rotate")
@click.argument("input_file")
@click.argument("output_file")
@click.option("--degrees", type=int, help="Rotate image by 90, 180 or 270 degrees")
def rotate_image_cli(input_file, output_file, degrees):
    """Rotate image by given degrees (90, 180, 270)"""
    try:
        rotate_image(input_file, output_file, degrees)
        click.echo(f"Rotated {input_file} to {output_file}")
    except Exception as e:
        click.echo(f"Error: {e}")


@manip.command("grayscale")
@click.argument("input_file")
@click.argument("output_file")
def grayscale_image_cli(input_file, output_file):
    """Convert image to grayscale"""
    try:
        grayscale_image(input_file, output_file)
        click.echo(f"Converted {input_file} to grayscale {output_file}")
    except Exception as e:
        click.echo(f"Error: {e}")


@cli.command()
@click.argument("file_path")
def info(file_path):
    """Displays info for a given file."""
    try:
        file_info = get_file_info(file_path)
        click.echo(f"File Info: \n{file_info}")
    except Exception as e:
        click.echo(f"Error: {e}")


if __name__ == "__main__":
    cli()



# file_mate/manipulators.py

from PIL import Image
from file_mate.utils import (
    validate_file_exists,
    validate_image_format,
    validate_output_dir,
)


def resize_image(input_file, output_file, width=None, height=None, scale=None):
    """Resizes an image to a specified width, height or scale

    Args:
         input_file (str): The input image file path
         output_file (str): The output image file path
         width (int, optional): The target image width in pixels. Defaults to None.
         height (int, optional): The target image height in pixels. Defaults to None.
         scale (float, optional): The scale factor to apply on the image (0-1) . Defaults to None.
    """

    validate_file_exists(input_file)
    validate_image_format(input_file)
    validate_output_dir(output_file)

    try:
        img = Image.open(input_file)
    except Exception as e:
        raise ValueError(f"Invalid image file: {e}")

    if scale:
        new_width = int(img.width * scale)
        new_height = int(img.height * scale)
    elif width and height:
        new_width = width
        new_height = height
    else:
        raise ValueError(
            f"Error: Either 'scale' or both 'width' and 'height' must be specified"
        )

    try:
        img = img.resize((new_width, new_height))
        img.save(output_file)
        img.close()  # Ensure the image file is closed
    except Exception as e:
        raise RuntimeError(f"Error resizing image: {e}")


def rotate_image(input_file, output_file, degrees):
    """Rotates an image by 90, 180 or 270 degrees.

    Args:
      input_file (str): The input image file path
      output_file (str): The output image file path
      degrees (int): The rotation amount in degrees. (90, 180, 270)
    """
    validate_file_exists(input_file)
    validate_image_format(input_file)
    validate_output_dir(output_file)

    try:
        img = Image.open(input_file)
        if degrees not in [90, 180, 270]:
            img.close()
            raise ValueError(
                f"Error: Rotation must be one of 90, 180 or 270. Got {degrees}"
            )
        rotated_img = img.rotate(-degrees, expand=True)
        rotated_img.save(output_file)
        img.close()
    except ValueError as e:
        raise e
    except Exception as e:
        raise RuntimeError(f"Error rotating image: {e}")


def grayscale_image(input_file, output_file):
    """Converts image to grayscale

    Args:
        input_file (str): The input image file path
        output_file (str): The output image file path
    """
    validate_file_exists(input_file)
    validate_image_format(input_file)
    validate_output_dir(output_file)

    try:
        img = Image.open(input_file).convert("L")
        img.save(output_file)
        img.close()  # Ensure the image file is closed

    except Exception as e:
        raise RuntimeError(f"Error grayscaling image: {e}")



# setup.py
from setuptools import setup, find_packages
import os


def get_long_description():
    with open(
        os.path.join(os.path.dirname(__file__), "README.md"), encoding="utf-8"
    ) as f:
        return f.read()


setup(
    name="file-mate",
    version="0.1.0",
    description="A command-line tool for image, pdf, and other file manipulations.",
    author="Anshul Gada",
    license="MIT",
    packages=find_packages(),
    install_requires=[
        "Pillow",
        "pypdf",
        "click",
        "reportlab",
        "python-magic-bin",
        "setuptools",
    ],
    entry_points={
        "console_scripts": [
            "file-mate=file_mate.cli:cli",
        ],
    },
    python_requires=">=3.8",
    package_data={
        "file_mate": ["tests/*"],
    },
)



# file_mate/info.py

import os
from file_mate.utils import validate_file_exists, get_file_type
from PIL import Image
from pypdf import PdfReader


def get_file_info(file_path):
    """Extracts basic file info such as type, size etc

    Args:
        file_path (str): The path of the input file.

    Returns:
        dict: File info dictionary.
    """
    validate_file_exists(file_path)
    file_type = get_file_type(file_path)

    file_size = os.path.getsize(file_path)
    info = {
        "type": file_type,
        "size_bytes": file_size,
        "size": format_file_size(file_size),
    }
    if "image" in file_type.lower():
        try:
            img = Image.open(file_path)
            info["dimensions"] = f"{img.width}x{img.height}"
        except Exception as e:
            info["dimensions"] = "N/A"
    elif "pdf" in file_type.lower():
        try:
            pdf = PdfReader(file_path)
            info["page_count"] = len(pdf.pages)
        except Exception as e:
            info["page_count"] = "N/A"

    return info


def format_file_size(size_bytes):
    """Formats bytes to a human-readable string."""
    if size_bytes < 1024:
        return f"{size_bytes} bytes"
    elif size_bytes < 1024 * 1024:
        return f"{size_bytes / 1024:.2f} KB"
    elif size_bytes < 1024 * 1024 * 1024:
        return f"{size_bytes / (1024 * 1024):.2f} MB"
    else:
        return f"{size_bytes / (1024 * 1024 * 1024):.2f} GB"
